@@ -1,5 +1,5 @@
 // App.cc
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -20,18 +20,12 @@
 // DEALINGS IN THE SOFTWARE.
 
 #include "App.hh"
+#include "FbString.hh"
 #include "Font.hh"
 #include "Image.hh"
 
 #include "EventManager.hh"
 
-#ifdef HAVE_CASSERT
-  #include <cassert>
-#else
-  #include <assert.h>
-#endif
-#include <string>
-
 namespace FbTk {
 
 App *App::s_app = 0;
@@ -54,16 +48,16 @@ App::App(const char *displayname):m_done(false), m_display(0) {
     if (!m_display)
         throw std::string("Couldn't connect to XServer");
 
-    Font::init();
+    FbStringUtil::init();
     Image::init();
 }
 
 App::~App() {
     if (m_display != 0) {
-        
+
         Font::shutdown();
         Image::shutdown();
-        
+
         XCloseDisplay(m_display);
         m_display = 0;
     }
@@ -87,4 +81,4 @@ void App::end() {
     m_done = true; //end loop in App::eventLoop
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
