@@ -1,5 +1,5 @@
 // EventManager.cc
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,15 +19,16 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: EventManager.cc 3864 2005-01-24 18:02:34Z mathias $
-
 #include "EventManager.hh"
+#include "EventHandler.hh"
 #include "FbWindow.hh"
 #include "App.hh"
-#include "I18n.hh"
 
+#ifdef DEBUG
 #include <iostream>
-using namespace std;
+using std::cerr;
+using std::endl;
+#endif // DEBUG
 
 namespace FbTk {
 
@@ -64,6 +65,26 @@ EventHandler *EventManager::find(Window win) {
     return m_eventhandlers[win];
 }
 
+bool EventManager::grabKeyboard(EventHandler &ev, Window win) {
+    if (m_grabbing_keyboard)
+        ungrabKeyboard();
+
+    int ret = XGrabKeyboard(App::instance()->display(), win, False,
+                            GrabModeAsync, GrabModeAsync, CurrentTime);
+
+    if (ret == Success) {
+        m_grabbing_keyboard = &ev;
+        return true;
+    }
+    return false;
+}
+
+void EventManager::ungrabKeyboard() {
+    XUngrabKeyboard(App::instance()->display(), CurrentTime);
+    if (m_grabbing_keyboard)
+        m_grabbing_keyboard->notifyUngrabKeyboard();
+    m_grabbing_keyboard = 0;
+}
 
 Window EventManager::getEventWindow(XEvent &ev) {
     // we only have cases for events that differ from xany
@@ -179,13 +200,13 @@ void EventManager::dispatch(Window win, XEvent &ev, bool parent) {
     break;
     };
 
-    // find out which window is the parent and 
+    // find out which window is the parent and
     // dispatch event
     Window root, parent_win, *children = 0;
     unsigned int num_children;
-    if (XQueryTree(FbTk::App::instance()->display(), win, 
+    if (XQueryTree(FbTk::App::instance()->display(), win,
                    &root, &parent_win, &children, &num_children) != 0) {
-        if (children != 0) 
+        if (children != 0)
             XFree(children);
 
         if (parent_win != 0 &&
@@ -200,4 +221,4 @@ void EventManager::dispatch(Window win, XEvent &ev, bool parent) {
 
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
