@@ -1,5 +1,5 @@
 // FbWindow.cc for FbTk - fluxbox toolkit
-// Copyright (c) 2002-2005 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,10 +19,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbWindow.cc 3835 2005-01-14 09:40:40Z mathias $
-
 #include "FbWindow.hh"
 #include "FbPixmap.hh"
+#include "FbString.hh"
 
 #include "EventManager.hh"
 #include "Color.hh"
@@ -44,20 +43,28 @@
 
 namespace FbTk {
 
-FbWindow::FbWindow():FbDrawable(), m_parent(0), m_screen_num(0), m_window(0), m_x(0), m_y(0),
-                     m_width(0), m_height(0), m_border_width(0), m_depth(0), m_destroy(true),
-                     m_buffer_pm(0){
+FbWindow::FbWindow():
+    FbDrawable(),
+    m_parent(0), m_screen_num(0), m_window(0),
+    m_x(0), m_y(0), m_width(0), m_height(0),
+    m_border_width(0), m_border_color(0),
+    m_depth(0), m_destroy(true),
+    m_lastbg_color_set(false), m_lastbg_color(0), m_lastbg_pm(0),
+    m_renderer(0) {
 
 }
 
-FbWindow::FbWindow(const FbWindow& the_copy):FbDrawable(),
-                                             m_parent(the_copy.parent()),
-                                             m_screen_num(the_copy.screenNumber()), m_window(the_copy.window()),
-                                             m_x(the_copy.x()), m_y(the_copy.y()),
-                                             m_width(the_copy.width()), m_height(the_copy.height()),
-                                             m_border_width(the_copy.borderWidth()),
-                                             m_depth(the_copy.depth()), m_destroy(true),
-                                             m_buffer_pm(0) {
+FbWindow::FbWindow(const FbWindow& the_copy):
+    FbDrawable(),
+    m_parent(the_copy.parent()),
+    m_screen_num(the_copy.screenNumber()), m_window(the_copy.window()),
+    m_x(the_copy.x()), m_y(the_copy.y()),
+    m_width(the_copy.width()), m_height(the_copy.height()),
+    m_border_width(the_copy.borderWidth()),
+    m_border_color(the_copy.borderColor()),
+    m_depth(the_copy.depth()), m_destroy(true),
+    m_lastbg_color_set(false), m_lastbg_color(0), m_lastbg_pm(0),
+    m_renderer(the_copy.m_renderer) {
     the_copy.m_window = 0;
 }
 
@@ -67,13 +74,15 @@ FbWindow::FbWindow(int screen_num,
                    long eventmask,
                    bool override_redirect,
                    bool save_unders,
-                   int depth,
+                   unsigned int depth,
                    int class_type):
     FbDrawable(),
     m_parent(0),
     m_screen_num(screen_num),
     m_destroy(true),
-    m_buffer_pm(0) {
+    m_lastbg_color_set(false),
+    m_lastbg_color(0),
+    m_lastbg_pm(0), m_renderer(0) {
 
     create(RootWindow(display(), screen_num),
            x, y, width, height, eventmask,
@@ -85,11 +94,12 @@ FbWindow::FbWindow(const FbWindow &parent,
                    long eventmask,
                    bool override_redirect,
                    bool save_unders,
-                   int depth, int class_type):
+                   unsigned int depth, int class_type):
     m_parent(&parent),
     m_screen_num(parent.screenNumber()),
     m_destroy(true),
-    m_buffer_pm(0) {
+    m_lastbg_color_set(false), m_lastbg_color(0),
+    m_lastbg_pm(0), m_renderer(0) {
 
     create(parent.window(), x, y, width, height, eventmask,
            override_redirect, save_unders, depth, class_type);
@@ -97,24 +107,24 @@ FbWindow::FbWindow(const FbWindow &parent,
 
 };
 
-FbWindow::FbWindow(Window client):FbDrawable(), m_parent(0),
-                                  m_screen_num(0),
-                                  m_window(0),
-                                  m_x(0), m_y(0),
-                                  m_width(1), m_height(1),
-                                  m_border_width(0),
-                                  m_depth(0),
-                                  m_destroy(false),  // don't destroy this window
-                                  m_buffer_pm(0) {
-
+FbWindow::FbWindow(Window client):
+    FbDrawable(),
+    m_parent(0), m_screen_num(0), m_window(0),
+    m_x(0), m_y(0), m_width(1), m_height(1),
+    m_border_width(0), m_border_color(0),
+    m_depth(0), m_destroy(false), // don't destroy this window
+    m_lastbg_color_set(false), m_lastbg_color(0), m_lastbg_pm(0),
+    m_renderer(0) {
     setNew(client);
 }
 
 FbWindow::~FbWindow() {
 
     // Need to free xrender pics before destroying window
-    if (m_transparent.get() != 0)
+    if (m_transparent.get() != 0) {
+        removeAlphaWin(*this);
         m_transparent.reset(0);
+    }
 
     if (m_window != 0) {
         // so we don't get any dangling eventhandler for this window
@@ -122,19 +132,114 @@ FbWindow::~FbWindow() {
         if (m_destroy)
             XDestroyWindow(display(), m_window);
     }
+
 }
 
-
 void FbWindow::setBackgroundColor(const FbTk::Color &bg_color) {
-    XSetWindowBackground(display(), m_window, bg_color.pixel());
+    if (bg_color.isAllocated()) {
+        m_lastbg_color = bg_color.pixel();
+        m_lastbg_color_set = true;
+        m_lastbg_pm = None;
+    } else {
+        m_lastbg_color_set = false;
+    }
+
+    updateBackground(false);
 }
 
 void FbWindow::setBackgroundPixmap(Pixmap bg_pixmap) {
-    XSetWindowBackgroundPixmap(display(), m_window, bg_pixmap);
+    m_lastbg_pm = bg_pixmap;
+    if (bg_pixmap != None)
+        m_lastbg_color_set = false;
+
+    updateBackground(false);
+}
+
+void FbWindow::invalidateBackground() {
+    m_lastbg_pm = None;
+    m_lastbg_color_set = false;
+}
+
+void FbWindow::updateBackground(bool only_if_alpha) {
+    Pixmap newbg = m_lastbg_pm;
+    unsigned char alpha = 255;
+    bool free_newbg = false;
+
+    if (m_lastbg_pm == None && !m_lastbg_color_set)
+        return;
+
+    if (m_transparent.get() != 0)
+        alpha = m_transparent->alpha();
+
+    if (only_if_alpha && alpha == 255) 
+        return;
+
+    // still use bg buffer pixmap if not transparent
+    // cause it does nice caching things, assuming we have a renderer
+    if (m_lastbg_pm != ParentRelative && (m_renderer || alpha != 255)) {
+        // update source and destination if needed
+        Pixmap root = FbPixmap::getRootPixmap(screenNumber());
+        if (alpha != 255 && m_transparent->source() != root)
+            m_transparent->setSource(root, screenNumber());
+
+        FbPixmap newpm = FbPixmap(*this, width(), height(), depth());
+        free_newbg = true; // newpm gets released to newbg at end of block
+        GC gc = XCreateGC(display(), window(), 0, 0);
+
+        if (m_lastbg_pm == None && m_lastbg_color_set) {
+            XSetForeground(display(), gc, m_lastbg_color);
+            newpm.fillRectangle(gc, 0, 0, width(), height());
+        } else {
+            // copy from window if no color and no bg...
+            newpm.copyArea((m_lastbg_pm == None)?drawable():m_lastbg_pm, gc, 0, 0, 0, 0, width(), height());
+        }
+        XFreeGC(display(), gc);
+
+        if (alpha != 255)
+            m_transparent->setDest(newpm.drawable(), screenNumber());
+
+        // get root position
+
+        const FbWindow *root_parent = parent();
+        // our position in parent ("root")
+        int root_x = x() + borderWidth(), root_y = y() + borderWidth();
+        if (root_parent != 0) {
+            root_x += root_parent->x() + root_parent->borderWidth();
+            root_y += root_parent->y() + root_parent->borderWidth();
+            while (root_parent->parent() != 0) {
+                root_parent = root_parent->parent();
+                root_x += root_parent->x() + root_parent->borderWidth();
+                root_y += root_parent->y() + root_parent->borderWidth();
+            }
+        }
+
+        // render background image from root pos to our window
+        if (alpha != 255)
+            m_transparent->render(root_x, root_y,
+                                  0, 0,
+                                  width(), height());
+
+        // render any foreground items
+        if (m_renderer)
+            m_renderer->renderForeground(*this, newpm);
+
+        if (alpha != 255)
+            m_transparent->freeDest(); // it's only temporary, don't leave it hanging around
+        newbg = newpm.release();
+    }
+
+    if (newbg != None)
+        XSetWindowBackgroundPixmap(display(), m_window, newbg);
+    else if (m_lastbg_color_set)
+        XSetWindowBackground(display(), m_window, m_lastbg_color);
+
+    if (free_newbg)
+        XFreePixmap(display(), newbg);
 }
 
 void FbWindow::setBorderColor(const FbTk::Color &border_color) {
     XSetWindowBorder(display(), m_window, border_color.pixel());
+    m_border_color = border_color.pixel();
 }
 
 void FbWindow::setBorderWidth(unsigned int size) {
@@ -152,15 +257,25 @@ void FbWindow::setEventMask(long mask) {
 
 void FbWindow::clear() {
     XClearWindow(display(), m_window);
+    if (m_lastbg_pm == ParentRelative && m_renderer)
+        m_renderer->renderForeground(*this, *this);
+
 }
 
 void FbWindow::clearArea(int x, int y,
                          unsigned int width, unsigned int height,
                          bool exposures) {
-    XClearArea(display(), window(), x, y, width, height, exposures);
+    // TODO: probably could call renderForeground here (with x,y,w,h)
+    if (m_lastbg_pm == ParentRelative && m_renderer)
+        FbWindow::clear();
+    else
+        XClearArea(display(), window(), x, y, width, height, exposures);
 }
 
-void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, unsigned int the_height) {
+// If override_is_offset, then dest_override is a pixmap located at the_x, the_y
+// with size the_width x the_height  in the target window.
+
+void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, unsigned int the_height, Pixmap dest_override, bool override_is_offset) {
 #ifdef HAVE_XRENDER
     if (!m_transparent.get())
         return;
@@ -168,6 +283,17 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
     if (width() == 0 || height() == 0)
         return;
 
+    if (!dest_override && ((the_width == 0 && the_height == 0) ||
+        (the_width == width() && the_height == height())) &&
+        the_x <= 0 && the_y <= 0) {
+        // do the whole thing
+        updateBackground(true);
+        return;
+    }
+
+    if (!dest_override)
+        dest_override = window();
+
     if (the_width == 0 || the_height == 0) {
         the_width = width();
         the_height = height();
@@ -183,13 +309,8 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
     if (m_transparent->source() != root)
         m_transparent->setSource(root, screenNumber());
 
-    if (m_buffer_pm) {
-        if (m_transparent->dest() != m_buffer_pm) {
-            m_transparent->setDest(m_buffer_pm, screenNumber());
-        }
-    } else if (m_transparent->dest() != window())
-        m_transparent->setDest(window(), screenNumber());
-
+    if (m_transparent->dest() != dest_override)
+        m_transparent->setDest(dest_override, screenNumber());
 
     // get root position
 
@@ -209,7 +330,7 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
 
     // render background image from root pos to our window
     m_transparent->render(root_x + the_x, root_y + the_y,
-                          the_x, the_y,
+                          override_is_offset?0:the_x, override_is_offset?0:the_y,
                           the_width, the_height);
 #endif // HAVE_XRENDER
 }
@@ -217,17 +338,27 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
 void FbWindow::setAlpha(unsigned char alpha) {
 #ifdef HAVE_XRENDER
     if (FbTk::Transparent::haveComposite()) {
-        if (m_transparent.get() != 0)
+        if (m_transparent.get() != 0) {
+            removeAlphaWin(*this);
             m_transparent.reset(0);
+        }
 
         // don't setOpaque, let controlling objects do that
         // since it's only needed on toplevel windows
-    } else if (m_transparent.get() == 0 && alpha < 255) {
-        m_transparent.reset(new Transparent(FbPixmap::getRootPixmap(screenNumber()), window(), alpha, screenNumber()));
-    } else if (alpha < 255 && alpha != m_transparent->alpha())
-        m_transparent->setAlpha(alpha);
-    else if (alpha == 255)
-        m_transparent.reset(0); // destroy transparent object
+    } else {
+        if (!FbTk::Transparent::haveRender()) 
+            alpha = 255;
+
+        if (m_transparent.get() == 0 && alpha < 255) {
+            m_transparent.reset(new Transparent(FbPixmap::getRootPixmap(screenNumber()), window(), alpha, screenNumber()));
+            addAlphaWin(*this);
+        } else if (alpha < 255 && alpha != m_transparent->alpha())
+            m_transparent->setAlpha(alpha);
+        else if (alpha == 255) {
+            removeAlphaWin(*this);
+            m_transparent.reset(0); // destroy transparent object
+        }
+    }
 #endif // HAVE_XRENDER
 }
 
@@ -248,6 +379,7 @@ FbWindow &FbWindow::operator = (const FbWindow &win) {
     m_width = win.width();
     m_height = win.height();
     m_border_width = win.borderWidth();
+    m_border_color = win.borderColor();
     m_depth = win.depth();
     // take over this window
     win.m_window = 0;
@@ -322,41 +454,70 @@ void FbWindow::setCursor(Cursor cur) {
     XDefineCursor(display(), window(), cur);
 }
 
-void FbWindow::unsetCursor() {
-    XUndefineCursor(display(), window());
-}
-
 void FbWindow::reparent(const FbWindow &parent, int x, int y, bool continuing) {
     XReparentWindow(display(), window(), parent.window(), x, y);
     m_parent = &parent;
     if (continuing) // we will continue managing this window after reparent
         updateGeometry();
 }
-
+struct TextPropPtr {
+    TextPropPtr(XTextProperty& prop):m_prop(prop) {}
+    ~TextPropPtr() {
+        if (m_prop.value != 0) {
+            XFree(m_prop.value);
+            m_prop.value = 0;
+        }
+    }
+    XTextProperty& m_prop;
+};
 std::string FbWindow::textProperty(Atom property) const {
     XTextProperty text_prop;
-    char ** stringlist;
-    int count;
+    TextPropPtr helper(text_prop);
+    char ** stringlist = 0;
+    int count = 0;
     std::string ret;
 
-    if (XGetTextProperty(display(), window(), &text_prop, property) == 0)
+    static Atom m_utf8string = XInternAtom(display(), "UTF8_STRING", False);
+
+    if (XGetTextProperty(display(), window(), &text_prop, property) == 0) {
         return "";
-
-    if (text_prop.value == 0 || text_prop.nitems == 0)
-        return "";
-
-    if (text_prop.encoding != XA_STRING) {
-        // still returns a "StringList" despite the different name
-        if (XmbTextPropertyToTextList(display(), &text_prop, &stringlist, &count) == 0 || count == 0)
-            return "";
-    } else {
-        if (XTextPropertyToStringList(&text_prop, &stringlist, &count) == 0 || count == 0)
-            return "";
-
     }
 
-    ret = stringlist[0];
-    XFreeStringList(stringlist);
+    if (text_prop.value == 0 || text_prop.nitems == 0) {
+        return "";
+    }
+
+    if (text_prop.encoding == XA_STRING) {
+        if (XTextPropertyToStringList(&text_prop, &stringlist, &count) == 0 || count == 0) {
+            return "";
+        }
+        ret = FbStringUtil::XStrToFb(stringlist[0]);
+    } else if (text_prop.encoding == m_utf8string && text_prop.format == 8) {
+#ifdef X_HAVE_UTF8_STRING
+        Xutf8TextPropertyToTextList(display(), &text_prop, &stringlist, &count);
+        if (count == 0 || stringlist == 0) {
+            return "";
+        }
+#else
+        if (XTextPropertyToStringList(&text_prop, &stringlist, &count) == 0 || count == 0 || stringlist == 0) {
+            return "";
+        }
+#endif
+        ret = stringlist[0];
+    } else {
+        // still returns a "StringList" despite the different name
+        XmbTextPropertyToTextList(display(), &text_prop, &stringlist, &count);
+        if (count == 0 || stringlist == 0) {
+            return "";
+        }
+        ret = FbStringUtil::LocaleStrToFb(stringlist[0]);
+    }
+
+    // they all use stringlist
+    if (stringlist) {
+        XFreeStringList(stringlist);
+    }
+
     return ret;
 }
 
@@ -394,6 +555,14 @@ void FbWindow::deleteProperty(Atom property) {
     XDeleteProperty(display(), m_window, property);
 }
 
+void FbWindow::addToSaveSet() {
+    XAddToSaveSet(display(), m_window);
+}
+
+void FbWindow::removeFromSaveSet() {
+    XRemoveFromSaveSet(display(), m_window);
+}
+
 int FbWindow::screenNumber() const {
     return m_screen_num;
 }
@@ -409,34 +578,31 @@ long FbWindow::eventMask() const {
 void FbWindow::setOpaque(unsigned char alpha) {
 #ifdef HAVE_XRENDER
     static Atom m_alphaatom = XInternAtom(display(), "_NET_WM_WINDOW_OPACITY", False);
-    unsigned int opacity = alpha << 24;
+    unsigned long opacity = alpha * 0x1010101;
     changeProperty(m_alphaatom, XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &opacity, 1l);
 #endif // HAVE_XRENDER
 }
 
-void FbWindow::setBufferPixmap(Pixmap pm) {
-    m_buffer_pm = pm;
-}
-
 void FbWindow::updateGeometry() {
     if (m_window == 0)
         return;
 
     Window root;
     unsigned int border_width, depth;
-    XGetGeometry(display(), m_window, &root, &m_x, &m_y,
-                 (unsigned int *)&m_width, (unsigned int *)&m_height,
-                 &border_width, &depth);
-    m_depth = depth;
+    if (XGetGeometry(display(), m_window, &root, &m_x, &m_y,
+                     (unsigned int *)&m_width, (unsigned int *)&m_height,
+                     &border_width, &depth))
+        m_depth = depth;
 }
 
 void FbWindow::create(Window parent, int x, int y,
                       unsigned int width, unsigned int height,
                       long eventmask, bool override_redirect,
-                      bool save_unders, int depth, int class_type) {
+                      bool save_unders, unsigned int depth, int class_type) {
 
 
     m_border_width = 0;
+    m_border_color = 0;
 
     long valmask = CWEventMask;
     XSetWindowAttributes values;
@@ -463,7 +629,52 @@ void FbWindow::create(Window parent, int x, int y,
     assert(m_window);
 
     updateGeometry();
-    FbWindow::setBackgroundColor(Color("gray", screenNumber()));
+}
+
+
+void FbWindow::sendConfigureNotify(int x, int y, 
+                                   unsigned int width, unsigned int height,
+                                   unsigned int bw) {
+    Display *disp = FbTk::App::instance()->display();
+    XEvent event;
+    event.type = ConfigureNotify;
+
+    event.xconfigure.display = disp;
+    event.xconfigure.event = window();
+    event.xconfigure.window = window();
+    event.xconfigure.x = x;
+    event.xconfigure.y = y;
+    event.xconfigure.width = width;
+    event.xconfigure.height = height;
+    event.xconfigure.border_width = bw;
+    event.xconfigure.above = None; 
+    event.xconfigure.override_redirect = false;
+
+    XSendEvent(disp, window(), False, StructureNotifyMask, &event);
+
+}
+
+FbWindow::FbWinList FbWindow::m_alpha_wins;
+
+void FbWindow::addAlphaWin(FbWindow &win) {
+    m_alpha_wins.insert(&win);
+}
+
+void FbWindow::removeAlphaWin(FbWindow &win) {
+    FbWinList::iterator it = m_alpha_wins.find(&win);
+    if (it != m_alpha_wins.end())
+        m_alpha_wins.erase(it);
+}
+
+void FbWindow::updatedAlphaBackground(int screen) {
+    FbWinList::iterator it = m_alpha_wins.begin();
+    FbWinList::iterator it_end = m_alpha_wins.end();
+    for (; it != it_end; ++it) {
+        if ((*it)->screenNumber() == screen) {
+            (*it)->updateBackground(false);
+            (*it)->clear();
+        }
+    }
 }
 
 bool operator == (Window win, const FbWindow &fbwin) {
