@@ -19,16 +19,13 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-/// $Id: WinButton.cc 4199 2006-02-16 06:53:05Z mathias $
-
-#include <X11/Xlib.h>
-
 #include "WinButton.hh"
-#include "App.hh"
 #include "Window.hh"
+#include "WindowCmd.hh"
 #include "Screen.hh"
 #include "WinClient.hh"
 #include "WinButtonTheme.hh"
+#include "FbTk/App.hh"
 #include "FbTk/Color.hh"
 
 #ifdef SHAPE
@@ -36,13 +33,15 @@
 #endif // SHAPE
 
 
-WinButton::WinButton(const FluxboxWindow &listen_to, 
-                     WinButtonTheme &theme,
+WinButton::WinButton(FluxboxWindow &listen_to,
+                     FbTk::ThemeProxy<WinButtonTheme> &theme,
+                     FbTk::ThemeProxy<WinButtonTheme> &pressed,
                      Type buttontype, const FbTk::FbWindow &parent,
                      int x, int y,
                      unsigned int width, unsigned int height):
     FbTk::Button(parent, x, y, width, height),
-    m_type(buttontype), m_listen_to(listen_to), m_theme(theme),
+    m_type(buttontype), m_listen_to(listen_to),
+    m_theme(theme), m_pressed_theme(pressed),
     m_icon_pixmap(0), m_icon_mask(0),
     overrode_bg(false), overrode_pressed(false) {
     theme.reconfigSig().attach(this);
@@ -57,13 +56,16 @@ void WinButton::exposeEvent(XExposeEvent &event) {
 }
 
 void WinButton::buttonReleaseEvent(XButtonEvent &event) {
+    WinClient *old = WindowCmd<void>::client();
+    WindowCmd<void>::setWindow(&m_listen_to);
     FbTk::Button::buttonReleaseEvent(event);
+    WindowCmd<void>::setClient(old);
 }
 
 // when someone else tries to set the background, we may override it
 void WinButton::setBackgroundPixmap(Pixmap pm) {
     Pixmap my_pm = getBackgroundPixmap();
-    
+
     if (my_pm != 0) {
         overrode_bg = true;
         pm = my_pm;
@@ -112,64 +114,30 @@ void WinButton::setPressedColor(const FbTk::Color &color) {
 }
 
 Pixmap WinButton::getBackgroundPixmap() const {
-    bool focused = m_listen_to.isFocused();
     switch(m_type) {
     case MAXIMIZE:
-        if (focused)
-            return m_theme.maximizePixmap().pixmap().drawable();
-        else
-            return m_theme.maximizeUnfocusPixmap().pixmap().drawable();
+        return m_theme->maximizePixmap().pixmap().drawable();
         break;
     case MINIMIZE:
-        if (focused)
-            return m_theme.iconifyPixmap().pixmap().drawable();
-        else
-            return m_theme.iconifyUnfocusPixmap().pixmap().drawable();
+        return m_theme->iconifyPixmap().pixmap().drawable();
         break;
     case STICK:
-        if (m_listen_to.isStuck()) {
-            if (focused)
-                return m_theme.stuckPixmap().pixmap().drawable();
-            else
-                return m_theme.stuckUnfocusPixmap().pixmap().drawable();
-        } else {
-            if (focused)
-                return m_theme.stickPixmap().pixmap().drawable();
-            else
-                return m_theme.stickUnfocusPixmap().pixmap().drawable();
-        }
+        if (m_listen_to.isStuck())
+            return m_theme->stuckPixmap().pixmap().drawable();
+        return m_theme->stickPixmap().pixmap().drawable();
         break;
     case CLOSE:
-        if (focused)
-            return m_theme.closePixmap().pixmap().drawable();
-        else
-            return m_theme.closeUnfocusPixmap().pixmap().drawable();
+        return m_theme->closePixmap().pixmap().drawable();
         break;
-    case SHADE: 
-        if (m_listen_to.isShaded()) {
-            if (focused)
-                return m_theme.unshadePixmap().pixmap().drawable();
-            else
-                return m_theme.unshadeUnfocusPixmap().pixmap().drawable();
-        } else {
-            if (focused)
-                return m_theme.shadePixmap().pixmap().drawable();
-            else
-                return m_theme.shadeUnfocusPixmap().pixmap().drawable();
-        }
+    case SHADE:
+        if (m_listen_to.isShaded())
+            return m_theme->unshadePixmap().pixmap().drawable();
+        return m_theme->shadePixmap().pixmap().drawable();
         break;
     case MENUICON:
-        if (m_icon_pixmap.drawable()) {
-            if (focused)
-                return m_theme.titleFocusPixmap().pixmap().drawable();
-            else
-                return m_theme.titleUnfocusPixmap().pixmap().drawable();
-        } else {
-            if (focused)
-                return m_theme.menuiconPixmap().pixmap().drawable();
-            else
-                return m_theme.menuiconUnfocusPixmap().pixmap().drawable();
-        }
+        if (m_icon_pixmap.drawable())
+            return m_theme->titlePixmap().pixmap().drawable();
+        return m_theme->menuiconPixmap().pixmap().drawable();
         break;
     }
     return None;
@@ -178,26 +146,23 @@ Pixmap WinButton::getBackgroundPixmap() const {
 Pixmap WinButton::getPressedPixmap() const {
     switch(m_type) {
     case MAXIMIZE:
-        return m_theme.maximizePressedPixmap().pixmap().drawable();
+        return m_pressed_theme->maximizePixmap().pixmap().drawable();
     case MINIMIZE:
-        return m_theme.iconifyPressedPixmap().pixmap().drawable();
+        return m_pressed_theme->iconifyPixmap().pixmap().drawable();
     case STICK:
-        return m_theme.stickPressedPixmap().pixmap().drawable();
+        return m_pressed_theme->stickPixmap().pixmap().drawable();
     case CLOSE:
-        return m_theme.closePressedPixmap().pixmap().drawable();
+        return m_pressed_theme->closePixmap().pixmap().drawable();
     case SHADE:
         if (m_listen_to.isShaded())
-            return m_theme.unshadePressedPixmap().pixmap().drawable();
+            return m_pressed_theme->unshadePixmap().pixmap().drawable();
         else
-            return m_theme.shadePressedPixmap().pixmap().drawable();
+            return m_pressed_theme->shadePixmap().pixmap().drawable();
     case MENUICON:
         if (m_icon_pixmap.drawable())
-            if (m_listen_to.isFocused())
-                return m_theme.titleFocusPixmap().pixmap().drawable();
-            else
-                return m_theme.titleUnfocusPixmap().pixmap().drawable();
+            return m_theme->titlePixmap().pixmap().drawable();
         else
-            return m_theme.menuiconPressedPixmap().pixmap().drawable();
+            return m_pressed_theme->menuiconPixmap().pixmap().drawable();
     }
     return None;
 }
@@ -245,7 +210,7 @@ void WinButton::drawType() {
         }
         break;
     case CLOSE:
-        drawLine(gc(), 
+        drawLine(gc(),
                  2, 2,
                  width() - 3, height() - 3);
         // I can't figure out why this second one needs a y offset of 1?????
@@ -259,46 +224,47 @@ void WinButton::drawType() {
         // XFree86 Version 4.3.0.1 (Debian 4.3.0.dfsg.1-1 20040428170728)
         // (X Protocol Version 11, Revision 0, Release 6.6)
 
-        drawLine(gc(), 
+        drawLine(gc(),
                  2, height() - 3,
                  width() - 3, 2);
         break;
     case SHADE:
-        
-        drawRectangle(gc(), 2, 2, width() - 5 - oddW, 2);
-        
-        XPoint points[3];
-        if (m_listen_to.isShaded()) {
-            points[1].x = (width() / 2) - 3; points[1].y = 7;
-            points[2].x = (width() / 2) + 4 - oddW; points[2].y = 7;
-            points[0].x = (width() / 2); points[0].y = height() / 2 + 2;
-        } else {
-            points[0].x = (width() / 2); points[0].y = 6;
-            points[1].x = (width() / 2) - 4; points[1].y = height() / 2 + 2;
-            points[2].x = (width() / 2) + 4 - oddW; points[2].y = height() / 2 + 2;
-        }
-        fillPolygon(gc(), points, 3, Convex, CoordModeOrigin);
+
+    {
+        int size = width() - 5 - oddW;
+
+        drawRectangle(gc(), 2, 2, size, 2);
+
+        // draw a one-quarter triangle below the rectangle
+        drawTriangle(gc(), (m_listen_to.isShaded() ?
+                            FbTk::FbDrawable::DOWN:
+                            FbTk::FbDrawable::UP),
+                     4, 6,
+                     size-2, size/2 - 1,
+                     100);
+
         break;
+    }
     case MENUICON:
         if (m_icon_pixmap.drawable()) {
 
             if (m_icon_mask.drawable()) {
-                XSetClipMask(m_listen_to.fbWindow().display(), 
+                XSetClipMask(m_listen_to.fbWindow().display(),
                              gc(), m_icon_mask.drawable());
-                XSetClipOrigin(m_listen_to.fbWindow().display(), 
+                XSetClipOrigin(m_listen_to.fbWindow().display(),
                              gc(), 2, 2);
             }
-            
+
             copyArea(m_icon_pixmap.drawable(),
                      gc(),
-                     0, 0, 
-                     2, 2, 
+                     0, 0,
+                     2, 2,
                      m_icon_pixmap.width(), m_icon_pixmap.height());
 
             if (m_icon_mask.drawable())
                 XSetClipMask(m_listen_to.fbWindow().display(), gc(), None);
         } else {
-            for (int y = height()/3; y <= height() - height()/3; y+=3) {
+            for (unsigned int y = height()/3; y <= height() - height()/3; y+=3) {
                 drawLine(gc(), width()/4, y, width() - width()/4 - oddW - 1, y);
             }
             drawRectangle(gc(),
@@ -310,77 +276,43 @@ void WinButton::drawType() {
 
 void WinButton::clear() {
     FbTk::Button::clear();
-    
-    // ensure the m_listen_to has actually a client
-    if (m_type == MENUICON && !m_listen_to.empty() && (
-        !m_icon_pixmap.drawable() ||
-        (m_icon_pixmap.width() != width() - 4 || 
-         m_icon_pixmap.height() != height() - 4))) {
-       
-        Display* display = m_listen_to.fbWindow().display();
-        int screen = m_listen_to.screen().screenNumber();
-        XWMHints* hints = XGetWMHints(display, m_listen_to.winClient().window());
-        if (hints == 0) {
-            m_icon_pixmap.release();
-            m_icon_mask.release();
-        } else {
-            if ((hints->flags & IconPixmapHint) && hints->icon_pixmap != 0) {
-                 m_icon_pixmap.copy(hints->icon_pixmap, 
-                                    DefaultDepth(display, screen), screen);
-                 m_icon_pixmap.scale(width() - 4, height() - 4);
-            } else
-                m_icon_pixmap.release();
-            
-            if ((hints->flags & IconMaskHint)) {
-                m_icon_mask.copy(hints->icon_mask, 0, 0);
-                m_icon_mask.scale(width() - 4, height() - 4);
-            } else
-                m_icon_mask.release();
-        }
-        
-        XFree(hints);
-    }
-
     drawType();
 }
 
 void WinButton::update(FbTk::Subject *subj) {
 
-    // just checking, if we the app provides a pixmap.
+    // update the menu icon
     if (m_type == MENUICON && !m_listen_to.empty()) {
-        XWMHints* hints = XGetWMHints(m_listen_to.fbWindow().display(), 
-                                      m_listen_to.winClient().window());
-        if (hints == 0) {
+
+        Display* display = m_listen_to.fbWindow().display();
+        int screen = m_listen_to.screen().screenNumber();
+        if (m_listen_to.icon().pixmap().drawable() != None) {
+             m_icon_pixmap.copy(m_listen_to.icon().pixmap().drawable(),
+                                DefaultDepth(display, screen), screen);
+             m_icon_pixmap.scale(width() - 4, height() - 4);
+        } else
             m_icon_pixmap.release();
+
+        if (m_listen_to.icon().mask().drawable() != None) {
+            m_icon_mask.copy(m_listen_to.icon().mask().drawable(), 0, 0);
+            m_icon_mask.scale(width() - 4, height() - 4);
+        } else
             m_icon_mask.release();
-        } else {
 
-            // no pixmap
-            if (!((hints->flags & IconPixmapHint) && hints->icon_pixmap != 0))
-                m_icon_pixmap.release();
-
-            // pixmap has changed
-            if (hints->flags & IconPixmapHint && hints->icon_pixmap != 0 && 
-                    hints->icon_pixmap != m_icon_pixmap.drawable())
-                m_icon_pixmap.release();
-            
-            // no pixmap-mask
-            if (!(hints->flags & IconMaskHint))
-                m_icon_mask.release();
-
-            // pixmap-mask has changed
-            if (hints->flags & IconMaskHint && hints->icon_mask != m_icon_mask.drawable())
-                m_icon_mask.release();
-        }
-        
-        XFree(hints);
     }
-    
+
     // pressed_pixmap isn't stateful in any current buttons, so no need
     // to potentially override that. Just make sure background pm is ok
     Pixmap my_pm = getBackgroundPixmap();
     if (my_pm != None)
         setBackgroundPixmap(my_pm);
 
+    // incorrect, pressed_pixmap is stateful in shade, so we'll do oneoff for now
+    if (m_type == SHADE) {
+        Pixmap p_pm = getPressedPixmap();
+        if (p_pm != None)
+            setPressedPixmap(p_pm);
+    }
+
     clear();
 }
