@@ -19,71 +19,77 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WinClient.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "WinClient.hh"
 
 #include "Window.hh"
 #include "fluxbox.hh"
+#include "FocusControl.hh"
 #include "Screen.hh"
 #include "FbAtoms.hh"
 
 #include "Xutil.hh"
 
-#include "EventManager.hh"
+#include "FbTk/EventManager.hh"
 #include "FbTk/MultLayers.hh"
 
 #include <iostream>
 #include <algorithm>
 #include <iterator>
 #include <memory>
+#include <X11/Xatom.h>
 
 #ifdef HAVE_CASSERT
   #include <cassert>
 #else
   #include <assert.h>
 #endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
-using namespace std;
+using std::string;
+using std::list;
+using std::mem_fun;
 
+#ifdef DEBUG
+using std::cerr;
+using std::endl;
+using std::hex;
+using std::dec;
+#endif // DEBUG
 
 WinClient::TransientWaitMap WinClient::s_transient_wait;
 
-WinClient::WinClient(Window win, BScreen &screen, FluxboxWindow *fbwin):FbTk::FbWindow(win),
+WinClient::WinClient(Window win, BScreen &screen, FluxboxWindow *fbwin):
+        Focusable(screen, fbwin), FbTk::FbWindow(win),
                      transient_for(0),
                      window_group(0),
-                     x(0), y(0), old_bw(0),
-                     min_width(1), min_height(1),
-                     max_width(0), max_height(0),
-                     width_inc(1), height_inc(1),
-                     min_aspect_x(0), min_aspect_y(0),
-                     max_aspect_x(0), max_aspect_y(0),
-                     base_width(1), base_height(1),
+                     old_bw(0),
                      initial_state(0),
                      normal_hint_flags(0),
                      wm_hint_flags(0),
-                     m_win(fbwin),
-                     m_modal(0),
+                     m_modal_count(0),
+                     m_modal(false),
+                     accepts_input(false),
                      send_focus_message(false),
                      send_close_message(false),
-                     m_win_gravity(0),
-                     m_title(""), m_icon_title(""),
-                     m_class_name(""), m_instance_name(""),
-                     m_blackbox_hint(0),
+                     m_title_override(false),
+                     m_icon_override(false),
+                     m_window_type(WindowState::TYPE_NORMAL),
                      m_mwm_hint(0),
-                     m_focus_mode(F_PASSIVE),
-                     m_diesig(*this), m_screen(screen), 
                      m_strut(0) {
+
+    old_bw = borderWidth();
     updateWMProtocols();
-    updateBlackboxHints();
     updateMWMHints();
     updateWMHints();
     updateWMNormalHints();
     updateWMClassHint();
     updateTitle();
-    updateIconTitle();
     Fluxbox::instance()->saveWindowSearch(win, this);
-    if (window_group != None) 
+    if (window_group != None)
         Fluxbox::instance()->saveGroupSearch(window_group, this);
 
     // search for this in transient waiting list
@@ -96,6 +102,11 @@ WinClient::WinClient(Window win, BScreen &screen, FluxboxWindow *fbwin):FbTk::Fb
         // clear transient waiting list for this window
         s_transient_wait.erase(win);
     }
+
+    // also check if this window is a transient
+    // this needs to be done before creating an fbwindow, so this doesn't get
+    // tabbed using the apps file
+    updateTransientInfo();
 }
 
 WinClient::~WinClient() {
@@ -107,35 +118,35 @@ WinClient::~WinClient() {
 
     clearStrut();
 
-    if (m_win != 0)
-        m_win->removeClient(*this);
-
-    // this takes care of any focus issues
-    m_diesig.notify();
-
-    Fluxbox *fluxbox = Fluxbox::instance();
-
-
     //
     // clear transients and transient_for
     //
     if (transient_for != 0) {
         assert(transient_for != this);
         transient_for->transientList().remove(this);
-        transient_for = 0;
+        if (m_modal)
+            transient_for->removeModal();
     }
-    
+
     while (!transients.empty()) {
         transients.back()->transient_for = 0;
         transients.pop_back();
     }
+
+    accepts_input = send_focus_message = false;
+    if (fbwindow() != 0)
+        fbwindow()->removeClient(*this);
+
+    // this takes care of any focus issues
+    m_diesig.notify();
+
     // This fixes issue 1 (see WinClient.hh):
     // If transients die before the transient_for is created
+    transient_for = 0;
     removeTransientFromWaitingList();
     s_transient_wait.erase(window());
 
-
-    screen().removeNetizen(window());
+    Fluxbox *fluxbox = Fluxbox::instance();
 
     if (window_group != 0) {
         fluxbox->removeGroupSearch(window_group);
@@ -145,22 +156,23 @@ WinClient::~WinClient() {
     if (m_mwm_hint != 0)
         XFree(m_mwm_hint);
 
-    if (m_blackbox_hint != 0)
-        XFree(m_blackbox_hint);
-
     if (window())
         fluxbox->removeWindowSearch(window());
-
-    m_win = 0;
 }
 
 bool WinClient::acceptsFocus() const {
-    return (m_focus_mode == F_LOCALLYACTIVE || 
-            m_focus_mode == F_PASSIVE || 
-            m_focus_mode == F_GLOBALLYACTIVE && send_focus_message);
+    return ((accepts_input || send_focus_message) &&
+            // focusing fbpanel messes up quite a few things
+            m_window_type != WindowState::TYPE_DOCK &&
+            m_window_type != WindowState::TYPE_SPLASH);
 }
 
 bool WinClient::sendFocus() {
+    if (accepts_input) {
+        setInputFocus(RevertToPointerRoot, CurrentTime);
+        FocusControl::setExpectingFocus(this);
+        return true;
+    }
     if (!send_focus_message)
         return false;
 #ifdef DEBUG
@@ -182,6 +194,7 @@ bool WinClient::sendFocus() {
     ce.xclient.data.l[4] = 0l;
     // send focus msg
     XSendEvent(display(), window(), false, NoEventMask, &ce);
+    FocusControl::setExpectingFocus(this);
     return true;
 }
 
@@ -216,15 +229,13 @@ bool WinClient::getWMName(XTextProperty &textprop) const {
 }
 
 bool WinClient::getWMIconName(XTextProperty &textprop) const {
-    return XGetWMName(display(), window(), &textprop);
+    return XGetWMIconName(display(), window(), &textprop);
 }
 
-const std::string &WinClient::getWMClassName() const {
-    return m_instance_name;
-}
-
-const std::string &WinClient::getWMClassClass() const {
-    return m_class_name;
+string WinClient::getWMRole() const {
+    Atom wm_role = XInternAtom(FbTk::App::instance()->display(),
+                               "WM_WINDOW_ROLE", False);
+    return textProperty(wm_role);
 }
 
 void WinClient::updateWMClassHint() {
@@ -233,15 +244,16 @@ void WinClient::updateWMClassHint() {
 #ifdef DEBUG
         cerr<<"WinClient: Failed to read class hint!"<<endl;
 #endif //DEBUG
-    } else {        
+        m_instance_name = m_class_name = "";
+    } else {
 
         if (ch.res_name != 0) {
             m_instance_name = const_cast<char *>(ch.res_name);
             XFree(ch.res_name);
             ch.res_name = 0;
-        } else 
+        } else
             m_instance_name = "";
-        
+
         if (ch.res_class != 0) {
             m_class_name = const_cast<char *>(ch.res_class);
             XFree(ch.res_class);
@@ -252,18 +264,13 @@ void WinClient::updateWMClassHint() {
 }
 
 void WinClient::updateTransientInfo() {
-#ifdef DEBUG
-    cerr<<__FUNCTION__<<": m_win = "<<m_win<<endl;
-#endif // DEBUG
-    if (m_win == 0)
-        return;
-
-
     // remove this from parent
     if (transientFor() != 0) {
         transientFor()->transientList().remove(this);
+        if (m_modal)
+            transientFor()->removeModal();
     }
-    
+
     transient_for = 0;
     // determine if this is a transient window
     Window win = 0;
@@ -281,24 +288,24 @@ void WinClient::updateTransientInfo() {
 #endif // DEBUG
         return;
     }
-	
-    if (win != None && m_win->screen().rootWindow() == win) {
+
+    if (win != None && screen().rootWindow() == win) {
         // transient for root window... =  transient for group
         // I don't think we are group-aware yet
-        return; 
+        return;
     }
 
 
     transient_for = Fluxbox::instance()->searchWindow(win);
     // if we did not find a transient WinClient but still
-    // have a transient X window, then we have to put the 
+    // have a transient X window, then we have to put the
     // X transient_for window in a waiting list and update this clients transient
     // list later when the transient_for has a Winclient
     if (!transient_for) {
         // We might also already waiting for an old transient_for;
-        // 
+        //
         // this call fixes issue 2:
-        // If transients changes to new transient_for before the old transient_for is created. 
+        // If transients changes to new transient_for before the old transient_for is created.
         // (see comment in WinClient.hh)
         //
         removeTransientFromWaitingList();
@@ -313,102 +320,55 @@ void WinClient::updateTransientInfo() {
 #endif // DEBUG
     // make sure we don't have deadlock loop in transient chain
     for (WinClient *w = this; w != 0; w = w->transient_for) {
-        if (w == w->transient_for) {
+        if (this == w->transient_for)
             w->transient_for = 0;
-            break;
-        }
     }
 
     if (transientFor() != 0) {
         // we need to add ourself to the right client in
         // the transientFor() window so we search client
         transient_for->transientList().push_back(this);
-
-        if (transientFor()->fbwindow() && transientFor()->fbwindow()->isStuck())
-            m_win->stick();
+        if (m_modal)
+            transient_for->addModal();
     }
 
 }
 
 
 void WinClient::updateTitle() {
-    // why 512? very very long wmnames seem to either 
+    // why 512? very very long wmnames seem to either
     // crash fluxbox or to make it have high cpuload
-    // see also: 
+    // see also:
     //    http://www.securityfocus.com/archive/1/382398/2004-11-24/2004-11-30/2
-    // 
+    //
     // TODO: - find out why this mostly happens when using xft-fonts
     //       - why other windowmanagers (pekwm/pwm3/openbox etc) are
     //         also influenced
     //
     // the limitation to 512 chars only avoids running in that trap
+    if (m_title_override)
+        return;
+
     m_title = string(Xutil::getWMName(window()), 0, 512);
+    titleSig().notify();
 }
 
-void WinClient::updateIconTitle() {
-    XTextProperty text_prop;
-    char **list = 0;
-    int num = 0;
-
-    if (getWMIconName(text_prop)) {
-        if (text_prop.value && text_prop.nitems > 0) {
-            if (text_prop.encoding != XA_STRING) {
-                text_prop.nitems = strlen((char *) text_prop.value);
-
-                if (XmbTextPropertyToTextList(display(), &text_prop,
-                                               &list, &num) == Success &&
-                    num > 0 && *list) {
-                    m_icon_title = (char *)*list;
-                    XFreeStringList(list);
-                } else
-                    m_icon_title = text_prop.value ? (char *)text_prop.value : "";
-            } else
-                m_icon_title = text_prop.value ? (char *)text_prop.value : "";
-
-            if (text_prop.value)
-                XFree((char *) text_prop.value);
-        } else
-            m_icon_title = title();
-    } else
-        m_icon_title = title();
-
+void WinClient::setTitle(const FbTk::FbString &title) {
+    m_title = title;
+    m_title_override = true;
+    titleSig().notify();
 }
 
-void WinClient::saveBlackboxAttribs(FluxboxWindow::BlackboxAttributes &blackbox_attribs) {
-    changeProperty(FbAtoms::instance()->getFluxboxAttributesAtom(),
-                   XA_CARDINAL, 32, PropModeReplace,
-                   (unsigned char *)&blackbox_attribs,
-                   FluxboxWindow::PropBlackboxAttributesElements
-        );
+void WinClient::setIcon(const FbTk::PixmapWithMask& pm) {
+
+    m_icon.pixmap().copy(pm.pixmap());
+    m_icon.mask().copy(pm.mask());
+    m_icon_override = true;
+    titleSig().notify();
 }
 
 void WinClient::setFluxboxWindow(FluxboxWindow *win) {
-    m_win = win;
-}
-
-void WinClient::updateBlackboxHints() {
-    int format;
-    Atom atom_return;
-    unsigned long num, len;
-    FbAtoms *atoms = FbAtoms::instance();
-
-    if (m_blackbox_hint) {
-        XFree(m_blackbox_hint);
-        m_blackbox_hint = 0;
-    }
-
-    if (property(atoms->getFluxboxHintsAtom(), 0,
-                 PropBlackboxHintsElements, False,
-                 atoms->getFluxboxHintsAtom(), &atom_return,
-                 &format, &num, &len,
-                 (unsigned char **) &m_blackbox_hint) &&
-        m_blackbox_hint) {
-
-        if (num != (unsigned)PropBlackboxHintsElements) {
-            XFree(m_blackbox_hint);
-            m_blackbox_hint = 0;
-        }
-    }
+    m_fbwin = win;
 }
 
 void WinClient::updateMWMHints() {
@@ -438,11 +398,10 @@ void WinClient::updateMWMHints() {
 
 void WinClient::updateWMHints() {
     XWMHints *wmhint = XGetWMHints(display(), window());
-    if (! wmhint) {
-        m_focus_mode = F_PASSIVE;
-        window_group = None;
-        initial_state = NormalState;
-    } else {
+    accepts_input = true;
+    window_group = None;
+    initial_state = NormalState;
+    if (wmhint) {
         wm_hint_flags = wmhint->flags;
         /*
          * ICCCM 4.1.7
@@ -455,34 +414,38 @@ void WinClient::updateWMHints() {
          * Globally Active   False         Present
          *---------------------------------------------
          * Here: WM_TAKE_FOCUS = send_focus_message
-         *       Input Field   = wmhint->input
-         *       Input Model   = m_focus_mode
+         *       Input Field   = accepts_input
          */
-        if (wmhint->flags & InputHint) {
-            if (wmhint->input) {
-                if (send_focus_message)
-                    m_focus_mode = F_LOCALLYACTIVE;
-                else
-                    m_focus_mode = F_PASSIVE;
-            } else {
-                if (send_focus_message)
-                    m_focus_mode = F_GLOBALLYACTIVE;
-                else
-                    m_focus_mode = F_NOINPUT;
-            }
-        } else // InputHint not present: ignoring send_focus_message and assuming F_PASSIVE
-            m_focus_mode = F_PASSIVE; 
+        if (wmhint->flags & InputHint)
+            accepts_input = (bool)wmhint->input;
 
         if (wmhint->flags & StateHint)
             initial_state = wmhint->initial_state;
-        else
-            initial_state = NormalState;
 
-        if (wmhint->flags & WindowGroupHint) {
-            if (! window_group)
-                window_group = wmhint->window_group;
-        } else
-            window_group = None;
+        if (wmhint->flags & WindowGroupHint && !window_group)
+            window_group = wmhint->window_group;
+
+        if (! m_icon_override) {
+
+            if ((bool)(wmhint->flags & IconPixmapHint) && wmhint->icon_pixmap != 0)
+                m_icon.pixmap().copy(wmhint->icon_pixmap, 0, 0);
+            else
+                m_icon.pixmap().release();
+
+            if ((bool)(wmhint->flags & IconMaskHint) && wmhint->icon_mask != 0)
+                m_icon.mask().copy(wmhint->icon_mask, 0, 0);
+            else
+                m_icon.mask().release();
+        }
+
+        if (fbwindow()) {
+            if (wmhint->flags & XUrgencyHint) {
+                Fluxbox::instance()->attentionHandler().addAttention(*this);
+            } else {
+                Fluxbox::instance()->attentionHandler().
+                    update(&m_focussig);
+            }
+        }
 
         XFree(wmhint);
     }
@@ -492,68 +455,11 @@ void WinClient::updateWMHints() {
 void WinClient::updateWMNormalHints() {
     long icccm_mask;
     XSizeHints sizehint;
-    if (! XGetWMNormalHints(display(), window(), &sizehint, &icccm_mask)) {
-        min_width = min_height =
-            base_width = base_height =
-            width_inc = height_inc = 1;
-        max_width = 0; // unbounded
-        max_height = 0;
-        min_aspect_x = min_aspect_y =
-            max_aspect_x = max_aspect_y = 0;
-        m_win_gravity = NorthWestGravity;
-    } else {
-        normal_hint_flags = sizehint.flags;
+    if (!XGetWMNormalHints(display(), window(), &sizehint, &icccm_mask))
+        sizehint.flags = 0;
 
-        if (sizehint.flags & PMinSize) {
-            min_width = sizehint.min_width;
-            min_height = sizehint.min_height;
-            if (!(sizehint.flags & PBaseSize)) {
-                base_width = min_width;
-                base_height = min_height;
-            }
-        } else {
-            min_width = min_height = 1;
-            base_width = base_height = 0;
-        }
-
-        if (sizehint.flags & PBaseSize) {
-            base_width = sizehint.base_width;
-            base_height = sizehint.base_height;
-            if (!(sizehint.flags & PMinSize)) {
-                min_width = base_width;
-                min_height = base_height;
-            }
-        } // default set in PMinSize
-
-        if (sizehint.flags & PMaxSize) {
-            max_width = sizehint.max_width;
-            max_height = sizehint.max_height;
-        } else {
-            max_width = 0; // unbounded
-            max_height = 0;
-        }
-
-        if (sizehint.flags & PResizeInc) {
-            width_inc = sizehint.width_inc;
-            height_inc = sizehint.height_inc;
-        } else
-            width_inc = height_inc = 1;
-
-        if (sizehint.flags & PAspect) {
-            min_aspect_x = sizehint.min_aspect.x;
-            min_aspect_y = sizehint.min_aspect.y;
-            max_aspect_x = sizehint.max_aspect.x;
-            max_aspect_y = sizehint.max_aspect.y;
-        } else
-            min_aspect_x = min_aspect_y =
-                max_aspect_x = max_aspect_y = 0;
-
-        if (sizehint.flags & PWinGravity)
-            m_win_gravity = sizehint.win_gravity;
-        else
-            m_win_gravity = NorthWestGravity;
-
-    }
+    normal_hint_flags = sizehint.flags;
+    m_size_hints.reset(sizehint);
 }
 
 Window WinClient::getGroupLeftWindow() const {
@@ -583,8 +489,10 @@ Window WinClient::getGroupLeftWindow() const {
 
 
 void WinClient::setGroupLeftWindow(Window win) {
+    if (m_screen.isShuttingdown())
+        return;
     Atom group_left_hint = XInternAtom(display(), "_FLUXBOX_GROUP_LEFT", False);
-    changeProperty(group_left_hint, XA_WINDOW, 32, 
+    changeProperty(group_left_hint, XA_WINDOW, 32,
                    PropModeReplace, (unsigned char *) &win, 1);
 }
 
@@ -613,16 +521,21 @@ bool WinClient::hasGroupLeftWindow() const {
     return false;
 }
 
-void WinClient::addModal() {
-    ++m_modal;
-    if (transient_for)
-        transient_for->addModal();
-}
+void WinClient::setStateModal(bool state) {
+    if (state == m_modal)
+        return;
 
-void WinClient::removeModal() {
-    --m_modal;
-    if (transient_for)
-        transient_for->removeModal();
+    m_modal = state;
+    if (transient_for) {
+        if (state)
+            transient_for->addModal();
+        else
+            transient_for->removeModal();
+    }
+
+    // TODO: we're not implementing the following part of EWMH spec:
+    // "if WM_TRANSIENT_FOR is not set or set to the root window the dialog is
+    //  modal for its window group."
 }
 
 bool WinClient::validateClient() const {
@@ -630,7 +543,7 @@ bool WinClient::validateClient() const {
 
     XEvent e;
     if (( XCheckTypedWindowEvent(display(), window(), DestroyNotify, &e) ||
-          XCheckTypedWindowEvent(display(), window(), UnmapNotify, &e)) 
+          XCheckTypedWindowEvent(display(), window(), UnmapNotify, &e))
         && XPutBackEvent(display(), &e)) {
         Fluxbox::instance()->ungrab();
         return false;
@@ -639,7 +552,7 @@ bool WinClient::validateClient() const {
     return true;
 }
 
-void WinClient::setStrut(Strut *strut) {    
+void WinClient::setStrut(Strut *strut) {
     clearStrut();
     m_strut = strut;
 }
@@ -653,10 +566,22 @@ void WinClient::clearStrut() {
 }
 
 bool WinClient::focus() {
-    if (m_win == 0)
+    if (fbwindow() == 0)
         return false;
     else
-        return m_win->setCurrentClient(*this, true);
+        return fbwindow()->setCurrentClient(*this, true);
+}
+
+bool WinClient::isFocused() const {
+    return (fbwindow() ?
+        fbwindow()->isFocused() && &fbwindow()->winClient() == this :
+        false);
+}
+
+void WinClient::setAttentionState(bool value) {
+    Focusable::setAttentionState(value);
+    if (fbwindow() && !fbwindow()->isFocused())
+        fbwindow()->setAttentionState(value);
 }
 
 void WinClient::updateWMProtocols() {
@@ -669,19 +594,16 @@ void WinClient::updateWMProtocols() {
         // defaults
         send_focus_message = false;
         send_close_message = false;
-        // could be added to netizens twice...
         for (int i = 0; i < num_return; ++i) {
             if (proto[i] == fbatoms->getWMDeleteAtom())
                 send_close_message = true;
             else if (proto[i] == fbatoms->getWMTakeFocusAtom())
                 send_focus_message = true;
-            else if (proto[i] == fbatoms->getFluxboxStructureMessagesAtom())
-                screen().addNetizen(window());
         }
 
         XFree(proto);
-        if (m_win)
-            m_win->updateFunctions();
+        if (fbwindow())
+            fbwindow()->updateFunctions();
 #ifdef DEBUG
     } else {
         cerr<<"Warning: Failed to read WM Protocols. "<<endl;
@@ -690,130 +612,13 @@ void WinClient::updateWMProtocols() {
 
 }
 
-/* For aspect ratios
-   Note that its slightly simplified in that only the
-   line gradient is given - this is because for aspect
-   ratios, we always have the line going through the origin
-   
-   * Based on this formula:
-   http://astronomy.swin.edu.au/~pbourke/geometry/pointline/
-
-   Note that a gradient from origin goes through ( grad , 1 )
- */
-
-void closestPointToLine(double &ret_x, double &ret_y, 
-                        double point_x, double point_y,
-                        double gradient) {
-    double u = (point_x * gradient + point_y) /
-        (gradient*gradient + 1);
-
-    ret_x = u*gradient;
-    ret_y = u;
-}
-
-/**
- * Changes width and height to the nearest (lower) value
- * that conforms to it's size hints.
- *
- * display_* give the values that would be displayed
- * to the user when resizing.
- * We use pointers for display_* since they are optional.
- *
- * See ICCCM section 4.1.2.3
- */
-void WinClient::applySizeHints(int &width, int &height, 
-                               int *display_width, int *display_height) {
-
-    int i = width, j = height;
-
-    // Check minimum size
-    if (width < 0 || width < static_cast<signed>(min_width)) 
-        width = min_width;
-
-    if (height < 0 || height < static_cast<signed>(min_height))
-        height = min_height;
-
-    // Check maximum size
-    if (max_width > 0 && width > static_cast<signed>(max_width))
-        width = max_width;
-
-    if (max_height > 0 && height > static_cast<signed>(max_height))
-        height = max_height;
-
-    // we apply aspect ratios before incrementals
-    // Too difficult to exactly satisfy both incremental+aspect 
-    // in most situations 
-    // (they really shouldn't happen at the same time anyway).
-
-    /* aspect ratios are applied exclusive to the base_width
-     *
-     * min_aspect_x      width      max_aspect_x
-     * ------------  <  -------  <  ------------
-     * min_aspect_y      height     max_aspect_y
-     *
-     * beware of integer maximum (so I'll use doubles instead and divide)
-     *
-     * The trick is how to get back to the aspect ratio with minimal
-     * change - do we modify x, y or both?
-     * A: we minimise the distance between the current point, and
-     *    the target aspect ratio (consider them as x,y coordinates)
-     *  Consider that the aspect ratio is a line, and the current
-     *  w/h is a point, so we're just using the formula for
-     *  shortest distance from a point to a line!
-     */
-
-    if (min_aspect_y > 0 && max_aspect_y > 0 &&
-        (height - base_height) > 0) {
-        double widthd = static_cast<double>(width - base_width);
-        double heightd = static_cast<double>(height - base_height);
-
-        double min = static_cast<double>(min_aspect_x) / 
-            static_cast<double>(min_aspect_y); 
-
-        double max = static_cast<double>(max_aspect_x) / 
-            static_cast<double>(max_aspect_y); 
-
-        double actual = widthd / heightd;
-
-        if (max > 0 && min > 0 && actual > 0) { // don't even try otherwise
-            bool changed = false;
-            if (actual < min) {
-                changed = true;
-                closestPointToLine(widthd, heightd, widthd, heightd, min);
-            } else if (actual > max) {
-                changed = true;
-                closestPointToLine(widthd, heightd, widthd, heightd, max);
-            }
-
-            if (changed) {
-                width = static_cast<int>(widthd) + base_width;
-                height = static_cast<int>(heightd) + base_height;
-            }
-        }
-    }
-
-    // enforce incremental size limits, wrt base size
-    // only calculate this if we really need to
-    i = (width - base_width) / width_inc;
-    width = i*width_inc + base_width;
-
-    j = (height - base_height) / height_inc;
-    height = j*height_inc + base_height;
-
-    if (display_width)
-        *display_width = i;
-
-    if (display_height)
-        *display_height = j;
-}
-
 void WinClient::removeTransientFromWaitingList() {
 
     // holds the windows that dont have empty
     // transient waiting list
-    std::list<Window> remove_list;
+    list<Window> remove_list;
 
-    // The worst case complexity is huge, but since we usually do not (virtualy never) 
+    // The worst case complexity is huge, but since we usually do not (virtualy never)
     // have a large transient waiting list the time spent here is neglectable
     TransientWaitMap::iterator t_it = s_transient_wait.begin();
     TransientWaitMap::iterator t_it_end = s_transient_wait.end();
@@ -826,8 +631,8 @@ void WinClient::removeTransientFromWaitingList() {
     }
 
     // erase empty waiting lists
-    std::list<Window>::iterator it = remove_list.begin();
-    std::list<Window>::iterator it_end = remove_list.end();
+    list<Window>::iterator it = remove_list.begin();
+    list<Window>::iterator it_end = remove_list.end();
     for (; it != it_end; ++it)
         s_transient_wait.erase(*it);
 }
