@@ -1,5 +1,5 @@
 // WorkspaceNameTool.cc
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,8 +20,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WorkspaceNameTool.cc 4046 2005-06-07 09:01:29Z mathias $
-
 #include "WorkspaceNameTool.hh"
 
 #include "ToolTheme.hh"
@@ -33,21 +31,20 @@
 #include <algorithm>
 
 WorkspaceNameTool::WorkspaceNameTool(const FbTk::FbWindow &parent, 
-                                     ToolTheme &theme, BScreen &screen):
+        FbTk::ThemeProxy<ToolTheme> &theme, BScreen &screen):
     ToolbarItem(ToolbarItem::FIXED),
-    m_button(parent, theme.font(), "a workspace name"),
+    m_button(parent, theme->font(), "a workspace name"),
     m_theme(theme),
     m_screen(screen),
     m_pixmap(0) {
     // set text
-    m_button.setGC(m_theme.textGC());
+    m_button.setGC(m_theme->textGC());
     m_button.setText(m_screen.currentWorkspace()->name());
 
     // setup signals
     screen.workspaceNamesSig().attach(this);
     screen.currentWorkspaceSig().attach(this);
     theme.reconfigSig().attach(this);
-
 }
 
 WorkspaceNameTool::~WorkspaceNameTool() {
@@ -82,12 +79,15 @@ void WorkspaceNameTool::update(FbTk::Subject *subj) {
 
 unsigned int WorkspaceNameTool::width() const {
     // calculate largest size
+    if (orientation() == FbTk::ROT90 || orientation() == FbTk::ROT270)
+        return m_button.width();
     unsigned int max_size = 0;
+
     const BScreen::Workspaces& workspaces = m_screen.getWorkspacesList();
     BScreen::Workspaces::const_iterator it;
     for (it = workspaces.begin(); it != workspaces.end(); it++) {
         const std::string &name = (*it)->name();
-        max_size = std::max(m_theme.font().textWidth(name.c_str(), name.size()), 
+        max_size = std::max(m_theme->font().textWidth(name, name.size()), 
                             max_size);
     }
     // so align text dont cut the last character
@@ -97,7 +97,21 @@ unsigned int WorkspaceNameTool::width() const {
 }
 
 unsigned int WorkspaceNameTool::height() const {
-    return m_button.height();
+    if (orientation() == FbTk::ROT0 || orientation() == FbTk::ROT180) 
+        return m_button.height();
+
+    unsigned int max_size = 0;
+    const BScreen::Workspaces& workspaces = m_screen.getWorkspacesList();
+    BScreen::Workspaces::const_iterator it;
+    for (it = workspaces.begin(); it != workspaces.end(); it++) {
+        const std::string &name = (*it)->name();
+        max_size = std::max(m_theme->font().textWidth(name, name.size()), 
+                            max_size);
+    }
+    // so align text dont cut the last character
+    max_size += 2;
+
+    return max_size;
 }
 
 unsigned int WorkspaceNameTool::borderWidth() const {
@@ -113,34 +127,35 @@ void WorkspaceNameTool::hide() {
 }
 
 void WorkspaceNameTool::updateSizing() {
-    m_button.setBorderWidth(m_theme.border().width());
+    m_button.setBorderWidth(m_theme->border().width());
 }
 
 void WorkspaceNameTool::reRender() {
-    if (m_theme.texture().usePixmap()) {
-        if (m_pixmap) 
-            m_screen.imageControl().removeImage(m_pixmap);
+    if (m_pixmap)
+        m_screen.imageControl().removeImage(m_pixmap);
+
+    if (m_theme->texture().usePixmap()) {
         m_pixmap = m_screen.imageControl().renderImage(width(), height(),
-                                                       m_theme.texture());
+                                                       m_theme->texture(), orientation());
         m_button.setBackgroundPixmap(m_pixmap);
+    } else {
+        m_pixmap = 0;
+        m_button.setBackgroundColor(m_theme->texture().color());
     }
 }
 
 void WorkspaceNameTool::renderTheme(unsigned char alpha) {
     
-    m_button.setJustify(m_theme.justify());
-    m_button.setBorderWidth(m_theme.border().width());
-    m_button.setBorderColor(m_theme.border().color());
+    m_button.setJustify(m_theme->justify());
+    m_button.setBorderWidth(m_theme->border().width());
+    m_button.setBorderColor(m_theme->border().color());
     m_button.setAlpha(alpha);
 
-    if (!m_theme.texture().usePixmap()) {
-        if (m_pixmap)
-            m_screen.imageControl().removeImage(m_pixmap);
-        m_pixmap = 0;
-        m_button.setBackgroundColor(m_theme.texture().color());
-    } else {
-        reRender();
-    }
-
+    reRender();
     m_button.clear();
 }
+
+void WorkspaceNameTool::setOrientation(FbTk::Orientation orient) {
+    m_button.setOrientation(orient);
+    ToolbarItem::setOrientation(orient);
+}
